<?php

namespace App\Controller;

use App\Entity\User;
use App\Form\User\SignUpFormType;
use App\Misc\FlashType;
use App\Repository\UserRepository;
use App\Service\ConnectedUserService;
use App\Service\LoggerService;
use Doctrine\ORM\EntityManagerInterface;
use LogicException;
use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Mailer\Exception\TransportExceptionInterface;
use Symfony\Component\Mime\Address;
use Symfony\Component\PasswordHasher\Hasher\UserPasswordHasherInterface;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Security\Http\Attribute\IsGranted;
use Symfony\Component\Security\Http\Authentication\AuthenticationUtils;
use Symfony\Contracts\Translation\TranslatorInterface;
use Throwable;

/**
 * Controllers for user pages
 */
#[Route('/user')]
class UserController extends AbstractController {
    /**
     * @var TranslatorInterface The translator service
     */
    private readonly TranslatorInterface $translator;
    /**
     * @var ConnectedUserService The connected user service
     */
    private readonly ConnectedUserService $connectedUserService;

    /**
     * Initialisation
     *
     * @param TranslatorInterface $translator The translator service
     * @param ConnectedUserService $connectedUserService The connected user service
     */
    public function __construct (TranslatorInterface $translator, ConnectedUserService $connectedUserService) {
        $this->translator = $translator;
        $this->connectedUserService = $connectedUserService;
    }

    /**
     * Register a new user
     *
     * @param Request $request The request
     * @param UserPasswordHasherInterface $userPasswordHasher The password hashing service
     * @param EntityManagerInterface      $entityManager      The entity manager
     *
     * @return Response The response
     *
     * @throws TransportExceptionInterface
     */
    #[Route('/signUp', name: 'user_signUp')]
    public function signUp (Request $request, UserPasswordHasherInterface $userPasswordHasher, EntityManagerInterface $entityManager): Response {
        if (($response = $this->connectedUserService->checkNotConnected())) {
            return $response;
        }

        $user = new User();

        $form = $this->createForm(SignUpFormType::class, $user);
        $form->handleRequest($request);

        if ($form->isSubmitted() && $form->isValid()) {
            //region Encode the plain password
            $user->setPassword(
                $userPasswordHasher->hashPassword(
                    $user,
                    $form->get('newPassword')->getData()
                )
            );
            //endregion

            $entityManager->persist($user);
            $entityManager->flush();

            $this->addFlash(FlashType::SUCCESS, 'Votre compte a bien été créé.');;
            return $this->redirectToRoute('user_signIn');
        }

        return $this->render('User/SignUp.html.twig', [
            'signUpForm' => $form->createView(),
        ]);
    }

    /**
     * Sign in a user
     *
     * @param AuthenticationUtils $authenticationUtils Security errors from query
     *
     * @return Response The response
     */
    #[Route(path: '/signIn', name: 'user_signIn')]
    public function login (AuthenticationUtils $authenticationUtils): Response {
        if (($response = $this->connectedUserService->checkNotConnected())) {
            return $response;
        }

        if (($error = $authenticationUtils->getLastAuthenticationError()) !== null) {
            $this->addFlash(FlashType::ERROR, $this->translator->trans($error->getMessageKey(), $error->getMessageData(), 'security'));
        }
        return $this->render(
            'User/SignIn.html.twig',
            [
                'last_username' => $authenticationUtils->getLastUsername(),
            ]
        );
    }

    /**
     * Sign out
     *
     * <b>NOTE :</b> dummy controller, intercepted by firewall
     *
     * @return void
     */
    #[Route(path: '/signOut', name: 'user_signOut')]
    public function logout (): void {
        throw new LogicException('This method can be blank - it will be intercepted by the logout key on your firewall.');
    }
}
