<?php

namespace App\Entity;

use App\Repository\RecipeRepository;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Constraints as Assert;

/**
 * A crafting recipe
 */
#[ORM\Entity(repositoryClass: RecipeRepository::class)]
class Recipe {
    use TBaseEntity;
    use TNamedEntity;

    /**
     * @var Machine|null The crafting machine
     */
    #[ORM\ManyToOne(inversedBy: 'recipes')]
    #[ORM\JoinColumn(nullable: false)]
    #[Assert\NotNull(message: 'Veuillez sélectionner une machine de fabrication')]
    #[Assert\Valid]
    private ?Machine $machine = null;
    /**
     * @var string|null The machine's first extra information
     */
    #[ORM\Column(length: 255, nullable: true)]
    #[Assert\NotBlank(message: 'Veuillez saisir l\'information complémentaire n° 1', allowNull: true)]
    private ?string $machineExtraInfo1 = null;
    /**
     * @var string|null The machine's second extra information
     */
    #[ORM\Column(length: 255, nullable: true)]
    #[Assert\NotBlank(message: 'Veuillez saisir l\'information complémentaire n° 2', allowNull: true)]
    private ?string $machineExtraInfo2 = null;
    /**
     * @var float|null The crafting time, in seconds
     */
    #[ORM\Column]
    #[Assert\Type(type: 'float', message: 'Le temps de production doit être un nombre réel')]
    #[Assert\NotNull(message: 'Veuillez saisir un temps de production')]
    #[Assert\Positive(message: 'Le temps de production doit être strictement positive')]
    private ?float $craftingTime = null;

    /**
     * @var Collection<int, InputRecipeMaterial> The consumed materials
     */
    #[ORM\OneToMany(targetEntity: InputRecipeMaterial::class, mappedBy: 'recipe', orphanRemoval: true)]
    #[Assert\Count(min: 1, minMessage: 'Veuillez ajouter au moins un matériau consommé')]
    #[Assert\Valid]
    private Collection $consumedMaterials;
    /**
     * @var Collection<int, OutputRecipeMaterial> The produced materials
     */
    #[ORM\OneToMany(targetEntity: OutputRecipeMaterial::class, mappedBy: 'recipe', orphanRemoval: true)]
    #[Assert\Count(min: 1, minMessage: 'Veuillez ajouter au moins un matériau produit')]
    #[Assert\Valid]
    private Collection $producedMaterials;

    /**
     * Initialization
     */
    public function __construct () {
        $this->consumedMaterials = new ArrayCollection();
        $this->producedMaterials = new ArrayCollection();
    }

    /**
     * The crafting machine
     *
     * @return Machine|null The crafting machine
     */
    public function getMachine (): ?Machine {
        return $this->machine;
    }
    /**
     * Set the crafting machine
     *
     * @param Machine|null $machine The crafting machine
     *
     * @return $this
     */
    public function setMachine (?Machine $machine): static {
        $this->machine = $machine;

        return $this;
    }

    /**
     * The machine's first extra information
     *
     * @return string|null The machine's first extra information
     */
    public function getMachineExtraInfo1 (): ?string {
        return $this->machineExtraInfo1;
    }
    /**
     * Set the machine's first extra information
     *
     * @param string|null $machineExtraInfo1 The machine's first extra information
     *
     * @return $this
     */
    public function setMachineExtraInfo1 (?string $machineExtraInfo1): self {
        $this->machineExtraInfo1 = $machineExtraInfo1;
        return $this;
    }
    /**
     * The machine's second extra information
     *
     * @return string|null The machine's second extra information
     */
    public function getMachineExtraInfo2 (): ?string {
        return $this->machineExtraInfo2;
    }
    /**
     * Set the machine's second extra information
     *
     * @param string|null $machineExtraInfo2 The machine's second extra information
     *
     * @return $this
     */
    public function setMachineExtraInfo2 (?string $machineExtraInfo2): self {
        $this->machineExtraInfo2 = $machineExtraInfo2;
        return $this;
    }

    /**
     * The crafting time, in seconds
     *
     * @return float|null The crafting time, in seconds
     */
    public function getCraftingTime (): ?float {
        return $this->craftingTime;
    }
    /**
     * Set the crafting time, in seconds
     *
     * @param float $craftingTime The crafting time, in seconds
     *
     * @return $this
     */
    public function setCraftingTime (float $craftingTime): static {
        $this->craftingTime = $craftingTime;

        return $this;
    }

    /**
     * The consumed materials
     *
     * @return Collection<int, InputRecipeMaterial> The consumed materials
     */
    public function getConsumedMaterials (): Collection {
        return $this->consumedMaterials;
    }
    /**
     * Add a consumed material
     *
     * @param InputRecipeMaterial $inputMaterial The consumed material
     *
     * @return $this
     */
    public function addConsumedMaterial (InputRecipeMaterial $inputMaterial): static {
        if (!$this->consumedMaterials->contains($inputMaterial)) {
            $this->consumedMaterials->add($inputMaterial);
            $inputMaterial->setRecipe($this);
        }

        return $this;
    }
    /**
     * Remove a consumed material
     *
     * @param InputRecipeMaterial $inputMaterial The consumed material
     *
     * @return $this
     */
    public function removeConsumedMaterial (InputRecipeMaterial $inputMaterial): static {
        if ($this->consumedMaterials->removeElement($inputMaterial)) {
            // set the owning side to null (unless already changed)
            if ($inputMaterial->getRecipe() === $this) {
                $inputMaterial->setRecipe(null);
            }
        }

        return $this;
    }

    /**
     * The produced materials
     *
     * @return Collection<int, OutputRecipeMaterial> The produced materials
     */
    public function getProducedMaterials(): Collection
    {
        return $this->producedMaterials;
    }
    /**
     * Add a produced material
     *
     * @param OutputRecipeMaterial $outputMaterial The produced material
     *
     * @return $this
     */
    public function addProducedMaterial(OutputRecipeMaterial $outputMaterial): static
    {
        if (!$this->producedMaterials->contains($outputMaterial)) {
            $this->producedMaterials->add($outputMaterial);
            $outputMaterial->setRecipe($this);
        }

        return $this;
    }
    /**
     * Remove a produced material
     *
     * @param OutputRecipeMaterial $outputMaterial The produced material
     *
     * @return $this
     */
    public function removeProducedMaterial(OutputRecipeMaterial $outputMaterial): static
    {
        if ($this->producedMaterials->removeElement($outputMaterial)) {
            // set the owning side to null (unless already changed)
            if ($outputMaterial->getRecipe() === $this) {
                $outputMaterial->setRecipe(null);
            }
        }

        return $this;
    }
}
