<?php

namespace App\Service;

use App\Entity\User;
use App\Misc\FlashType;
use Symfony\Bundle\SecurityBundle\Security;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Session\Session;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Security\Core\Exception\AccessDeniedException;

/**
 * Service for the connected user
 */
readonly class ConnectedUserService {
    /**
     * @var Security The security service
     */
    public Security $security;
    /**
     * @var RouterInterface The routing service
     */
    private RouterInterface $router;
    /**
     * @var RequestStack The service for the request
     */
    private RequestStack $request;

    /**
     * Initialization
     *
     * @param Security                   $security   The security service
     * @param RouterInterface            $router     The routing service
     * @param RequestStack               $request    The service for the request
     */
    public function __construct (Security $security, RouterInterface $router, RequestStack $request) {
        $this->security = $security;
        $this->router = $router;
        $this->request = $request;
    }

    /**
     * The connected user
     *
     * @return User|null The connected user
     */
    public function getUser (): ?User {
        $user = $this->security->getUser();
        if ($user === null) {
            return null;
        }
        if (!$user instanceof User) {
            return null;
        }
        return $user;
    }

    /**
     * Is the requested user the connected one ?
     *
     * @param User $requestedUser The requested user
     *
     * @return bool True if the requested user is the connected one, else false
     */
    public function isRequestedUser (User $requestedUser): bool {
        return $this->getUser()?->getId() !== $requestedUser->getId();
    }

    /**
     * Check the user is NOT connected
     *
     * @return Response|null The response (warning and redirect) if user is connected, else Null
     */
    public function checkNotConnected (): ?Response {
        /** @var User|null $user */
        $user = $this->getUser();
        if ($user === null) {
            return null;
        }

        /** @var Session $session */
        $session = $this->request->getSession();
        $session->getFlashBag()->add(
            FlashType::WARNING,
            "Vous êtes déjà connecté, merci de vous <a href=\"{$this->router->generate('user_signOut')}\">déconnecter</a> d'abord"
        );
        return new RedirectResponse($this->router->generate('core_main'), 302);
    }
    /**
     * Check if the requested user is the connected one or if the last has administration privileges
     *
     * @param User $requestedUser The requested user
     *
     * @return void
     *
     * @throws AccessDeniedException If the access is denied
     */
    public function checkRequestedUserAccess (User $requestedUser): void {
        if (!$this->isRequestedUser($requestedUser)) {
            return;
        }
        if (!$this->security->isGranted('ROLE_ADMIN')) {
            return;
        }

        $exception = new AccessDeniedException();
        $exception->setAttributes(['ROLE_ADMIN']);
        $exception->setSubject(null);

        throw $exception;
    }
}